unit MonitorWin;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, JvExExtCtrls, JvShape;

type
  TMonitorWindow = class(TForm)
    JvShape2: TJvShape;
    JvShape1: TJvShape;
    JvShape3: TJvShape;
    JvShape4: TJvShape;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    FirstMoveX, FirstMoveY : Integer;
    Movable                : Boolean;
    StayOnTop              : Boolean;
    ForceStayOnTop         : Boolean;
  public
  end;

var
  MonitorWindow: TMonitorWindow;

implementation

{$R *.dfm}

procedure TMonitorWindow.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := FALSE;
end;

procedure ForceOnTop;
begin
  SetThreadPriority(GetCurrentThread, THREAD_PRIORITY_BELOW_NORMAL);

  repeat
    Sleep(64);

    SetWindowPos(MonitorWindow.Handle, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE or SWP_NOSIZE or SWP_NOACTIVATE or SWP_NOOWNERZORDER);
  until 1 <> 1;
end;

procedure TMonitorWindow.FormCreate(Sender: TObject);
var
  tid : Cardinal;
begin
  Left := Screen.Width - ClientWidth;
  Top := 0;

  Movable := TRUE;
  StayOnTop := TRUE;
  ForceStayOnTop := TRUE;

  If StayOnTop Then
    MonitorWindow.FormStyle := fsStayOnTop
  else
    MonitorWindow.FormStyle := fsNormal;

  SetWindowLong(Application.Handle, GWL_EXSTYLE, WS_EX_TOOLWINDOW);

  If ForceStayOnTop Then
    BeginThread(nil, 0, @ForceOnTop, nil, 0, tid);
end;

procedure TMonitorWindow.FormMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  If (Movable) and
     (not Assigned(MonitorWindow.OnMouseMove)) and
     (Button = mbLeft) Then
  Begin
    MonitorWindow.OnMouseMove := FormMouseMove;
    FirstMoveX := X;
    FirstMoveY := Y;
  End;
end;

procedure TMonitorWindow.FormMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  MonitorWindow.ScreenSnap := not (ssCtrl in Shift);
    
  MonitorWindow.Left := MonitorWindow.Left - FirstMoveX + X;
  MonitorWindow.Top := MonitorWindow.Top - FirstMoveY + Y;
end;

procedure TMonitorWindow.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  If (Assigned(MonitorWindow.OnMouseMove)) and
     (Button = mbLeft) Then
    MonitorWindow.OnMouseMove := nil;
end;

end.
